global k_APPNAME	:= "Build Script Lib"
global k_EXIT_ENV	:= 1
global k_EXIT_RUN	:= 2


clean() {
	; Remove REALbasic temp files that end up left over
	FileSetAttrib, -H, .temp*, 0
	FileDelete, .temp*
	
	; Make sure these don’t get copied into the source archive
	FileDelete, *.exe
	FileDelete, *.zip
}


buildPortable(programName, extraFiles := "") {
	Progress, A M T ZX32 ZY16 FS8, Initialising, Building %programName% portable, Build portable
	
	sevenZipPath := get7ZipPath()
	if (!sevenZipPath) {
		msgBox 16, %k_APPNAME%, 7-Zip cannot be found on this system.
		ExitApp, %k_EXIT_ENV%
	}
	
	Progress, 0, Creating directory %programName%\
	FileCreateDir, %programName%
	
	Progress, 2, Copying file %programName%.exe
	FileCopy, %programName%.exe, %programName%, 1
	
	Progress, 48, Copying file %programName% licence.txt
	FileCopy, %programName% licence.txt, %programName%, 1
	
	Progress, 50, Creating %programName%\%programName% preferences.xml
	FileAppend, , %programName%\%programName% preferences.xml
	
	if (extraFiles) {
		nextProgress := 52
		progressIncrement := Floor((80 - 52) / extraFiles.Length())
		for index, fileName in extraFiles {
			Progress, %nextProgress%, Copying extra file %fileName%
			nextProgress += progressIncrement
			
			FileCopy, %fileName%, %programName%, 1
		}
	}
	
	Progress, 80, Compressing to %programName%.zip
	
	FileDelete, %programName%.zip
	cmd := """" . sevenZipPath . "\7zG.exe"" a """ . programName . ".zip"" """ . programName . "\"""
	RunWait, % cmd, , UseErrorLevel
	if (ErrorLevel == "ERROR") {
		msgBox 16, %k_APPNAME%, % "Invocation of 7-Zip failed; command was: " . cmd
		ExitApp, %k_EXIT_RUN%
	}
	else if (ErrorLevel) {
		ExitApp, %k_EXIT_RUN%
	}
	
	Progress, 98, Removing work directory
	FileRemoveDir, %programName%, 1
	
	Progress, 100, Build complete
	Sleep, 500
	
	Progress, Off
}


makeSourceArchive(programName, extraLibraries := "") {
	Progress, A M T ZX32 ZY16 FS8, Initialising, Building %programName% source archive, Build portable
	
	libraries := [ "Telcontar" ]
	if (extraLibraries) {
		for index, libName in extraLibraries {
			libraries.Insert(libName)
		}
	}
	
	sevenZipPath := get7ZipPath()
	if (!sevenZipPath) {
		msgBox 16, %k_APPNAME%, Project directory cannot be found.7-Zip cannot be found on this system.
		ExitApp, %k_EXIT_ENV%
	}
	
	projectDir := ""
	if (FileExist("..\Build") && FileExist("..\Libraries"))
		projectDir := ".."
	else if (FileExist("..\..\Build") && FileExist("..\..\Libraries"))
		projectDir := "..\.."
	else {
		msgBox 16, %k_APPNAME%, % "Project directory tree (containing Libraries and Build) cannot be found; this should be one or two levels above the program source directory."
		ExitApp, %k_EXIT_ENV%
	}
	
	Progress, 0, Creating directory tree %programName% source\
	buildDir := A_Temp . "\BuildScriptLib_" . A_Now
	FileCreateDir, %buildDir%
	Progress, 1
	sourceDir := buildDir . "\" . programName . " source"
	FileCreateDir, %sourceDir%
	Progress, 2
	FileCreateDir, %sourceDir%\Libraries
	
	Progress, 5, Copying central build files
	FileCopyDir, %projectDir%\Build, %sourceDir%\Build
	
	Progress, 10, Copying application files
	FileCopyDir, %A_ScriptDir%, %sourceDir%\Application
	
	nextProgress := 20
	progressIncrement := Floor((80 - 20) / libraries.Length())
	for index, libName in libraries {
		Progress, %nextProgress%, Copying library %libName%
		nextProgress += progressIncrement
		
		FileCopyDir, %projectDir%\Libraries\%libName%, %sourceDir%\Libraries\%libName%
	}
	
	Progress, 80, Creating %programName% source.zip
	
	cmd := """" . sevenZipPath . "\7zG.exe"" a """ . programName . " source.zip"" """ . sourceDir . "\"" -xr!"".DS_Store"""
	RunWait, % cmd, , UseErrorLevel
	if (ErrorLevel == "ERROR") {
		msgBox 16, %k_APPNAME%, % "Invocation of 7-Zip failed; command was: " . cmd
		ExitApp, %k_EXIT_RUN%
	}
	else if (ErrorLevel) {
		ExitApp, %k_EXIT_RUN%
	}
	
	Progress, 98, Removing work directory
	FileRemoveDir, %buildDir%, 1
	
	Progress, 100, Build complete
	Sleep, 500
}


get7ZipPath() {
	; Locate the installation of 7-Zip on this computer and return the path containing it
	; More detection methods may need to be added in future :)
	; Taken from 7-Ade
		
	sevenZipPath := ""
	
	if (A_Is64bitOS)
		SetRegView, 64
	
	RegRead, sevenZipPath, HKEY_CURRENT_USER, SOFTWARE\7-Zip, Path
	if (!sevenZipPath)
	{
		RegRead, sevenZipPath, HKEY_LOCAL_MACHINE, SOFTWARE\7-Zip, Path
	}
	
	if (sevenZipPath && substr(sevenZipPath, -1) == "\")
		sevenZipPath := substr(sevenZipPath, 0, -1)
	
	if (A_Is64bitOS)
		SetRegView, Default
	
	return sevenZipPath
}